class code:
    def __init__(self, code = 0, cmd_bytes = 0, reply_bytes = 0):
        self.code = code
        self.cmd_bytes = cmd_bytes
        self.reply_bytes = reply_bytes

# General Commands

NO_OP = code(0x00, 0, 0)
SET_DEFAULTS = code(0x01, 0, 0)
CAMERA_RESET = code(0x02, 0, 0)
RESTORE_FACTORY_DEFAULTS = code(0x03, 0, 0)
SERIAL_NUMBER = code(0x04, 0, 8)
GET_REVISION = code(0x05, 0, 8)

# Gain Commands

GET_GAIN_MODE = code(0x0A, 0, 2)
SET_GAIN_MODE = code(0x0A, 2, 2)

# FFC Commands

GET_FFC_MODE = code(0x0B, 0, 2)
SET_FFC_MODE = code(0x0B, 2, 2)

GET_FFC_NFRAMES = code(0x0B, 4, 2)
SET_FFC_NFRAMES = code(0x0B, 4, 2)

DO_FFC_SHORT = code(0x0C, 0, 0)
DO_FFC = code(0x0C, 2, 2)

GET_FFC_PERIOD = code(0x0D, 0, 4)
SET_FFC_PERIOD_LOW_GAIN = code(0x0D, 2, 2)
SET_FFC_PERIOD_HIGH_GAIN = code(0x0D, 2, 2)
SET_FFC_PERIOD = code(0x0D, 4, 4)

GET_FFC_TEMP_DELTA = code(0x0E, 0, 4)
SET_FFC_TEMP_DELTA_LOW_GAIN = code(0x0E, 2, 2)
SET_FFC_TEMP_DELTA_HIGH_GAIN = code(0x0E, 2, 2)
SET_FFC_TEMP_DELTA = code(0x0E, 4, 4)

# Video Mode Commands
GET_VIDEO_MODE = code(0x0F, 0, 2)
SET_VIDEO_MODE = code(0x0F, 2, 2)

GET_VIDEO_SYMBOLOGY_DIGITAL = code(0x0F, 4, 2)
SET_VIDEO_SYMBOLOGY_DIGITAL = code(0x0F, 4, 4)
GET_VIDEO_SYMBOLOGY_ANALOG = code(0x0F, 4, 2)
SET_VIDEO_SYMBOLOGY_ANALOG = code(0x0F, 4, 4)

GET_VIDEO_PALETTE = code(0x10, 0, 2)
SET_VIDEO_PALETTE = code(0x10, 2, 2)

GET_VIDEO_ORIENTATION = code(0x11, 0, 2)
SET_VIDEO_ORIENTATION = code(0x11, 2 ,2)

GET_DIGITAL_OUTPUT_MODE = code(0x12, 0, 2)
SET_DIGITAL_OUTPUT_MODE = code(0x12, 2, 2)

SET_CONTRAST = code(0x14, 0, 2)
GET_CONTRAST = code(0x14, 2, 2)

SET_BRIGHTNESS = code(0x15, 0, 2)
GET_BRIGHTNESS = code(0x15, 2, 2)

SET_BRIGHTNESS_BIAS = code(0x18, 0, 2)
GET_BRIGHTNESS_BIAS = code(0x18, 2, 2)

SET_AGC_TAIL_SIZE = code(0x1B, 0, 2)
GET_AGC_TAIL_SIZE = code(0x1B, 2, 2)

SET_AGC_ACE_CORRECT = code(0x1C, 0, 2)
GET_AGC_ACE_CORRECT = code(0x1C, 2, 2)

# AGC

GET_AGC_ALGORITHM = code(0x13, 0, 2)
SET_AGC_ALGORITHM = code(0x13, 2, 2)

GET_AGC_THRESHOLD = code(0x13, 2, 2)
SET_AGC_THRESHOLD = code(0x13, 4, 0)

GET_AGC_OPTIMISATION_PERCENT = code(0x13, 2, 2)
SET_AGC_OPTIMISATION_PERCENT = code(0x13, 4, 0)

# Lens

SET_LENS_NUMBER = code(0x1E, 0, 2)
GET_LENS_NUMBER = code(0x1E, 2, 2)

GET_LENS_GAIN_SWITCH = code(0x1E, 2, 2)
SET_LENS_GAIN_SWITCH = code(0x1E, 4, 4)

GET_LENS_GAIN_MAPPING = code(0x1E, 2, 2)
SET_LENS_GAIN_MAPPING = code(0x1E, 4, 4)

# Spot Meter

SET_SPOT_METER_MODE = code(0x1F, 0, 2)
GET_SPOT_METER_MODE = code(0x1F, 2, 2)

# Onboard sensors

READ_SENSOR_TEMPERATURE = code(0x20, 2, 2)
READ_SENSOR_ACCELEROMETER = code(0x20, 2, 8)
READ_SENSOR_STATUS = code(0x20, 2, 2)

# Sync

GET_EXTERNAL_SYNC = code(0x21, 0, 2)
SET_EXTERNAL_SYNC = code(0x21, 2, 2)

# Isotherm

GET_ISOTHERM = code(0x22, 0, 2)
SET_ISOTHERM = code(0x22, 2, 2)

GET_ISOTHERM_THRESHOLD = code(0x23, 0, 6)
SET_ISOTHERM_THRESHOLD = code(0x23, 6, 6)

GET_ISOTHERM_THRESHOLD_FOUR = code(0x23, 4, 4)
SET_ISOTHERM_THRESHOLD_FOUR = code(0x23, 4, 4)

# Test Pattern

GET_TEST_PATTERN = code(0x25, 0, 2)
SET_TEST_PATTERN = code(0x25, 2, 2)

SET_VIDEO_COLOR_MODE = code(0x26, 0, 2)
GET_VIDEO_COLOR_MODE = code(0x26, 2, 2)

# Shutter

GET_SHUTTER_POSITION = code(0x79, 0, 2)
SET_SHUTTER_POSITION = code(0x79, 2, 2)

# Frame transfer

TRANSFER_FRAME = code(0x82, 4, 4)

GET_MEMORY_ADDRESS = code(0xD6, 4, 8)
GET_NV_MEMORY_SIZE = code(0xD5, 2, 8)

READ_MEMORY_256 = code(0xD2, 6, 256)

ERASE_BLOCK = code(0xD4, 2, 2)

MEMORY_STATUS = code(0xC4, 0, 2)

# Radiometry

GET_PLANCK_COEFFICIENTS = code(0xB9, 0, 16)
SET_PLANCK_COEFFICIENTS = code(0xB9, 18, 18)


from flirpy.io.fff import Fff
from flirpy.util.exiftool import Exiftool
import os

class TestFff:
    def setup_method(self):

        folder = os.path.dirname(os.path.abspath(__file__))
        self.test_data_fff = os.path.join(folder, "test_data", "frame_000000.fff")
        self.test_data_meta = os.path.join(folder, "test_data", "frame_000000.txt")

        self.frame = Fff(self.test_data_fff)

    def test_read_image(self):
        image = self.frame.get_image()
        assert len(image.shape) == 2

    def test_read_radiometric_image(self):
        meta = Exiftool().meta_from_file(self.test_data_meta)
        image = self.frame.get_radiometric_image(meta)
        assert len(image.shape) == 2

    def test_fff_gps(self):
        coord =  self.frame.get_gps()
        assert len(coord) == 9
    
    def test_from_bytes(self):

        with open(self.test_data_fff, 'rb') as infile:
            self.frame = Fff(infile.read())

            # Check GPS for sanity
            coord =  self.frame.get_gps()
            assert len(coord) == 9



import cv2
from flirpy.camera.core import Core
import struct
import ctypes
import binascii
from serial.tools import list_ports
import os
import sys
import logging

# FFC Mode enum
FLR_BOSON_MANUAL_FFC = 0
FLR_BOSON_AUTO_FFC = 1
FLR_BOSON_EXTERNAL_FFC = 2
FLR_BOSON_SHUTTER_TEST_FFC = 3
FLR_BOSON_FFCMODE_END = 4

# FFC State enum
FLR_BOSON_NO_FFC_PERFORMED = 0
FLR_BOSON_FFC_IMMINENT = 1
FLR_BOSON_FFC_IN_PROGRESS = 2
FLR_BOSON_FFC_COMPLETE = 3
FLR_BOSON_FFCSTATUS_END = 4

crc_table = [
   0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,
   0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF,
   0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,
   0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE,
   0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,
   0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE, 0xF5CF, 0xC5AC, 0xD58D,
   0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,
   0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D, 0xC7BC,
   0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
   0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B,
   0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,
   0xDBFD, 0xCBDC, 0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A,
   0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,
   0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD, 0xAD2A, 0xBD0B, 0x8D68, 0x9D49,
   0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,
   0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A, 0x9F59, 0x8F78,
   0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,
   0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
   0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,
   0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
   0xB5EA, 0xA5CB, 0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,
   0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
   0xA7DB, 0xB7FA, 0x8799, 0x97B8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,
   0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
   0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9, 0xB98A, 0xA9AB,
   0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
   0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,
   0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
   0xFD2E, 0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9,
   0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
   0xEF1F, 0xFF3E, 0xCF5D, 0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8,
   0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
]
import subprocess
import pkg_resources

class Boson(Core):

    def __init__(self, port=None, baudrate=921600, loglevel=logging.DEBUG):
        self.command_count = 0
        self.cap = None
        self.conn = None

        logging.basicConfig(level=loglevel)
        self.logger = logging.getLogger(__name__)

        if port is None:
            port = self.find_serial_device()
            if port is not None:
                self.connect(port, baudrate)

                if self.conn.is_open:
                    print("Connected")

    def find_serial_device(self):
        """
        Attempts to find and return the serial port that the Boson is connected to.
        
        Returns: The port, if found. Returns None if not found.
        """
        port = None

        device_list = list_ports.comports()

        VID = 0x09CB
        PID = 0x4007
        
        for device in device_list:
            if device.vid == VID and device.pid == PID:
                port = device.device
                break

        return port

    def find_video_device(self):
        """
        Attempts to automatically detect which video device corresponds to the Boson by searching for the PID and VID.

        Returns: device number, or None if device not found.
        """

        res = None

        if sys.platform.startswith('win32'):
            device_check_path = pkg_resources.resource_filename('flirpy', 'bin/find_cameras.exe')
            device_id = int(subprocess.check_output([device_check_path, "FLIR Video"]).decode())

            if device_id >= 0:
                return device_id

        elif sys.platform == "darwin":
            output = subprocess.check_output(["system_profiler", "SPCameraDataType"]).decode()
            devices = [line.strip() for line in output.decode().split("\n") if line.strip().startswith("Model")]

            device_id = 0

            for device in devices:
                if device.contains("VendorID_2507") and device.contains("ProductID_16391"):
                    return device_id
            
        else:
            path = "/sys/class/video4linux/"
            devices = [os.path.join(path, device) for device in os.listdir(path)]
            
            for device in devices:
                with open(os.path.join(device, "name"), 'r') as d:
                    device_name = d.read()

                    if device_name == "Boson: FLIR Video\n":
                        return int(device[-1])

        return res
        
    def setup_video(self, device_id=None):
        """
        Setup the camera for video/frame capture.

        Attempts to automatically locate the camera, then opens an OpenCV VideoCapture object. The
        capture object is setup to capture raw video.
        """

        if device_id is None:
            device_id = self.find_video_device()
        
        if device_id is None:
            raise ValueError("Boson not connected.")

        if sys.platform.startswith('linux'):
            self.cap = cv2.VideoCapture(device_id + cv2.CAP_V4L2)
        elif sys.platform.startswith('win32'):
            self.cap = cv2.VideoCapture(device_id + cv2.CAP_DSHOW)
        else:
            # Catch anything else, e.g. Mac?
            self.cap = cv2.VideoCapture(device_id)

        if not self.cap.isOpened():
           raise IOError("Failed to open capture device")
        
        # The order of these calls matters!
        self.cap.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc(*"Y16 "))
        self.cap.set(cv2.CAP_PROP_CONVERT_RGB, False)
        

    def grab(self, device_id=None):
        """
        Captures and returns an image.

        Returns: the captured image as an array, or None if an error occured.
        """

        if self.cap is None:
            self.setup_video(device_id)
        
        return self.cap.read()[1]

    def reboot(self):
        """
        Reboot the camera
        """
        function_id = 0x00050010
        self._send_packet(function_id)

        return 


    def get_sensor_serial(self):
        """
        Get the serial number of the sensor
        """
        function_id = 0x00050006

        res = self._send_packet(function_id, receive_size=4)
        res = self._decode_packet(res, receive_size=4)

        return struct.unpack(">I", res)[0]
    
    def get_firmware_revision(self):
        """
        Get the camera's software revision.

        Returns: (Major, Minor, Patch)
        """

        function_id = 0x00050022

        res = self._send_packet(function_id, receive_size=12)
        res = self._decode_packet(res, receive_size=12)

        return struct.Struct(">iii").unpack_from(res)
    
    def get_part_number(self):

        function_id = 0x00050004
        res = self._send_packet(function_id, receive_size=20)
        res = self._decode_packet(res, receive_size=20)

        return res.decode("utf-8") 

    def do_ffc(self):
        """
        Manually request a flat field correction (FFC)
        """
        function_id = 0x00050007
        self._send_packet(function_id)

        return 
        
    def get_ffc_state(self):
        """
        Returns the FFC state:

        0 = FLR_BOSON_NO_FFC_PERFORMED
        1 = FLR_BOSON_FFC_IMMINENT
        2 = FLR_BOSON_FFC_IN_PROGRESS
        3 = FLR_BOSON_FFC_COMPLETE
        4 = FLR_BOSON_FFCSTATUS_END

        These return values are available as e.g.:

        flirpy.camera.boson.FLR_BOSON_NO_FFC_PERFORMED

        """
        function_id = 0x0005000C

        res = self._send_packet(function_id, receive_size=2)
        res = self._decode_packet(res, receive_size=2)

        return struct.unpack(">H", res)[0]

    def get_ffc_mode(self):
        """
        Get the current FFC mode

        0 = FLR_BOSON_NO_FFC_PERFORMED
        1 = FLR_BOSON_FFC_IMMINENT
        2 = FLR_BOSON_FFC_IN_PROGRESS
        3 = FLR_BOSON_FFC_COMPLETE
        4 = FLR_BOSON_FFCSTATUS_END
        """
        function_id = 0x00050013
        res = self._send_packet(function_id, receive_size=4)
        res = self._decode_packet(res, receive_size=4)

        return struct.unpack(">I", res)[0]

    def set_ffc_auto(self):
        """
        Set the FFC mode to automatic
        """
        function_id = 0x00050012
        command = struct.pack(">I", FLR_BOSON_AUTO_FFC)
        res = self._send_packet(function_id, data=command)
        res = self._decode_packet(res)

        return

    def set_ffc_manual(self):
        """
        Set the FFC mode to manual
        """
        function_id = 0x00050012
        command = struct.pack(">I", FLR_BOSON_MANUAL_FFC)
        res = self._send_packet(function_id, data=command)
        res = self._decode_packet(res)

        return
    
    def set_ffc_temperature_threshold(self, temp_difference):
        """
        Set the change in camera temperature required before an FFC is requested.
        """
        function_id = 0x00050008
        command = struct.pack(">H", int(temp_difference * 10))
        res = self._send_packet(function_id, data=command)
        res = self._decode_packet(res)

        return
    
    def get_ffc_temperature_threshold(self):
        """
        Get the change in camera temperature before an FFC is requested

        Returns: temperature in Celsius
        """
        function_id = 0x00050009

        res = self._send_packet(function_id, receive_size=2)
        res = self._decode_packet(res, receive_size=2)

        return struct.unpack(">H", res)[0]/10.0
    
    def set_ffc_frame_threshold(self, seconds):
        """
        Set the number of seconds before an FFC is requested.
        """
        function_id = 0x0005000A
        command = struct.pack(">I", seconds)
        res = self._send_packet(function_id, data=command)
        res = self._decode_packet(res)

        return
    
    def get_ffc_frame_threshold(self):
        """
        Get the number of frames before an FFC is requested.
        """
        function_id = 0x0005000B
        res = self._send_packet(function_id, receive_size=4)
        res = self._decode_packet(res, receive_size=4)

        return struct.unpack(">I", res)[0]
    
    def get_last_ffc_temperature(self):
        """
        Get the temperature (in Kelvin) that the last FFC occured
        """
        function_id = 0x0005005E

        res = self._send_packet(function_id, receive_size=2)
        res = self._decode_packet(res, receive_size=2)

        return struct.unpack(">H", res)[0]/10.0


    def get_last_ffc_frame_count(self):
        """
        Get the frame count when the last FFC occured
        """

        function_id = 0x0005005D

        res = self._send_packet(function_id, receive_size=4)
        res = self._decode_packet(res, receive_size=4)

        return struct.unpack(">I", res)[0]
    
    def get_frame_count(self):
        """
        Get the number of frames captured since the camera was turned on.

        Returns: number of frames
        """
        function_id = 0x00020002

        res = self._send_packet(function_id, receive_size=4)
        res = self._decode_packet(res, receive_size=4)

        return struct.unpack(">I", res)[0]

    def get_fpa_temperature(self):
        """
        Get the current focal plane array (FPA) temperature in Celsius.

        Returns: temperature in Celsius
        """
        function_id = 0x00050030

        res = self._send_packet(function_id, receive_size=2)
        res = self._decode_packet(res, receive_size=2)

        return struct.unpack(">H", res)[0]/10.0

    def get_camera_serial(self):
        """
        Get the camera serial number 

        Returns: serial number as an integer
        """
        function_id = 0x00050002

        res = self._send_packet(function_id, receive_size=4)
        res = self._decode_packet(res, receive_size=4)

        return struct.unpack(">I", res)[0]

    def _crc(self, data):
        """
        Full CRC with escape bytes (ugh)
        """
        pass
    
    def _decode_packet(self, data, receive_size=0):
        """
        Decodes a data packet from the camera.
        """

        payload = None

        if receive_size > 0:
            frame = struct.Struct(">BBIII{}sHB".format(receive_size))
            res = frame.unpack(data)

            start_marker, channel_id, sequence, function_id, return_code, payload, crc, end_marker = res
        else:
            frame = struct.Struct(">BBIIIHB")
            res = frame.unpack(data)

            start_marker, channel_id, sequence, function_id, return_code, crc, end_marker = res
            

        if start_marker != 0x8E and end_marker != 0xAE:
            self.logger.warning("Invalid frame markers")

        crc_bytes = binascii.crc_hqx(data[1:-3], 0x1D0F)

        if crc != crc_bytes:
            self.logger.warning("Invalid checksum")
        
        return payload

    def _send_packet(self, function_id, data=None, receive_size=0):
        """
        Sends a data packet to the camera.

        Send Package Format:

        Start frame byte = 0x8E
        Channel ID = 0
        Bytes 0:3 - sequence number
        Bytes 4:7 - function ID
        Bytes 8:11 - 0xFFFFFFFF
        Bytes 12: - payload (optional)
        CRC bytes - unsigned 16-bit CRC
        End frame byte = 0xAE
        """

        start_frame = 0x8E
        end_frame = 0xAE
        channel_number = 0x00

        header = struct.Struct(">BBIII")
        header_bytes = bytearray(header.pack(start_frame,
                                channel_number,
                                self.command_count,
                                function_id,
                                0xFFFFFFFF))

        payload = header_bytes

        if data is not None:
            payload += bytes(data)

        # The CRC is computed from the channel number to the last data byte.
        # This is actually incorrect, since we need to escape certain characters
        # in the payload (e.g. the start/end markers). But this will work for 
        # simple commands.
        crc_bytes = binascii.crc_hqx(payload[1:], 0x1D0F)

        footer = struct.Struct(">HB")
        footer_bytes = footer.pack(crc_bytes, end_frame)
        payload += footer_bytes

        self.send(payload)
        res = self.receive(17+receive_size)

        return res



import numpy as np
import cv2
import struct
import os
import re
import mmap
import tqdm
import logging
from glob import iglob, glob
import subprocess
from tqdm.autonotebook import tqdm

try:
  from pathlib import Path
except ImportError:
  from pathlib2 import Path  # python 2 backport

from flirpy.util.exiftool import Exiftool
from flirpy.io.fff import Fff

logger = logging.getLogger(__name__)

class splitter:
    
    def __init__(self, output_folder="./", exiftool_path=None, start_index=0, step=1, width=640, height=512, split_folders=True):
        
        self.exiftool = Exiftool(exiftool_path)
            
        self.width = width
        self.height = height
        self.start_index = start_index
        self.step = step
        self.frame_count = self.start_index
        self.export_tiff = True
        self.export_meta = True
        self.export_preview = True
        self.export_radiometric = True
        self.overwrite = True
        self.split_folders = True
        self.use_mmap = True

        self.output_folder = os.path.expanduser(output_folder)
        Path(self.output_folder).mkdir(exist_ok=True)
    
    def set_start_index(self, index):
        self.start_index = int(index)
        
    def process(self, file_list):

        if isinstance(file_list, str):
            file_list = [file_list]

        file_list = [os.path.expanduser(f) for f in file_list]

        logger.info("Splitting {} files".format(len(file_list)))
        
        self.frame_count = self.start_index
        
        for seq in tqdm(file_list):
            subfolder, _ = os.path.splitext(os.path.basename(seq))
            folder = os.path.join(self.output_folder, subfolder)
            Path(folder).mkdir(exist_ok=True)

            logger.info("Splitting {} into {}".format(seq, folder))
            self._process_seq(seq, folder)

            # Batch export meta data
            if self.export_meta:
                logger.info("Extracting metadata")

                if self.split_folders:
                    filemask = os.path.join(folder, "raw", "frame_*.fff")
                    copy_filemask = os.path.normpath("./raw/%f.fff")
                    radiometric_folder = os.path.normpath("./radiometric")
                else:
                    filemask = os.path.join(folder, "frame_*.fff")
                    copy_filemask = os.path.normpath("%f.fff")
                    radiometric_folder = os.path.normpath("./")

                self.exiftool.write_meta(filemask)

                # Copy geotags
                self.exiftool.copy_meta(folder, filemask=copy_filemask,output_folder=radiometric_folder)
        
    def _write_tiff(self, filename, data):
        cv2.imwrite(filename, data.astype("uint16"))

    def _write_preview(self, filename, data):
        drange = data.max()-data.min()
        preview_data = 255.0*((data-data.min())/drange)
        cv2.imwrite(filename, preview_data.astype('uint8'))
            
    def _make_split_folders(self, output_folder):
        Path(os.path.join(output_folder, "raw")).mkdir(exist_ok=True)
        Path(os.path.join(output_folder, "radiometric")).mkdir(exist_ok=True)
        Path(os.path.join(output_folder, "preview")).mkdir(exist_ok=True)

    def _get_fff_iterator(self, seq_blob):
        
        magic_pattern_fff = "\x46\x46\x46\x00".encode()

        valid = re.compile(magic_pattern_fff)
        return valid.finditer(seq_blob)

    def _check_overwrite(self, path):
        exists = os.path.exists(path)
        return (not exists) or (exists and self.overwrite)
    
    def _process_seq(self, input_file, output_subfolder):
        
        logger.debug("Processing {}".format(input_file))
        
        with open(input_file, 'rb') as seq_file:
            
            # Memory mapping may speed up things. This is kinda slow though, because we still have to parse the entire file
            # and then go back through the regexes to find individual frames. Should really use a stream.
            if self.use_mmap:
                seq_blob = mmap.mmap(seq_file.fileno(), 0, access=mmap.ACCESS_READ)
            else:
                seq_blob = seq_file.read()

            it = self._get_fff_iterator(seq_blob)

            pos = []
            prev_pos = 0
            
            meta = None
            
            for i, match in tqdm(enumerate(it)):
                index = match.start()
                chunksize = index-prev_pos
                pos.append((index, chunksize))
                prev_pos = index
                
                if self.split_folders:
                    self._make_split_folders(output_subfolder)
                                     
                    filename_fff = os.path.join(output_subfolder, "raw", "frame_{0:06d}.fff".format(self.frame_count))
                    filename_tiff = os.path.join(output_subfolder, "radiometric", "frame_{0:06d}.tiff".format(self.frame_count))
                    filename_preview = os.path.join(output_subfolder, "preview", "frame_{0:06d}.png".format(self.frame_count))
                    filename_meta = os.path.join(output_subfolder, "raw", "frame_{0:06d}.txt".format(self.frame_count))
                else:
                    filename_fff = os.path.join(output_subfolder, "frame_{0:06d}.fff".format(self.frame_count))
                    filename_tiff = os.path.join(output_subfolder, "frame_{0:06d}.tiff".format(self.frame_count))
                    filename_preview = os.path.join(output_subfolder, "frame_{0:06d}.png".format(self.frame_count))
                    filename_meta = os.path.join(output_subfolder, "frame_{0:06d}.txt".format(self.frame_count))
                
                if index == 0:
                    continue
                
                # Extract next FFF frame
                if self.use_mmap is False:
                    chunk = seq_blob[index:index+chunksize]
                else:
                    chunk = seq_blob.read(chunksize)
                
                if i % self.step == 0:

                    frame = Fff(chunk)
                    
                    # Need FFF files to extract meta, but we do it one go afterwards
                    if self.export_meta and self._check_overwrite(filename_fff):
                            frame.write(filename_fff)
                    
                    # We need at least one meta file to get the radiometric conversion coefficients
                    if meta is None and self.export_radiometric:
                        frame.write(filename_fff)
                        self.exiftool.write_meta(filename_fff)
                        meta = self.exiftool.meta_from_file(filename_meta)

                    # Export raw files and/or radiometric convert them
                    if self.export_tiff and self._check_overwrite(filename_tiff):
                            if self.export_radiometric and meta is not None:
                                image = frame.get_radiometric_image(meta)
                                image += 273.15 # Convert to Kelvin
                                image /= 0.04
                            else:
                                image = frame.get_image()

                            self._write_tiff(filename_tiff, image)

                    # Export preview frame (crushed to 8-bit)
                    if self.export_preview and self._check_overwrite(filename_preview):
                        self._write_preview(filename_preview, image)
            
                self.frame_count += 1
                    
        return


from flirpy.camera.boson import Boson
import flirpy.camera.boson
import pytest
import os
import time

with Boson() as camera:
    if camera.conn is None:
        pytest.skip("Boson not connected, skipping tests", allow_module_level=True)

def test_open_boson():
   camera = Boson()
   camera.close()

def test_get_serial():
    with Boson() as camera:
        assert camera.get_camera_serial() != 0

@pytest.mark.skip
def test_get_sensor_serial():
    # Skipped until proper CRC is implemented
    with Boson() as camera:
        assert camera.get_sensor_serial() != 0

def test_get_firmware_revision():
    with Boson() as camera:
        rev = camera.get_firmware_revision()
        assert len(rev) == 3

def test_get_fpa_temperature():
    with Boson() as camera:
        # In principle this could be exactly zero, but it's quite unlikely.
        assert camera.get_fpa_temperature() != 0

def test_get_part_number():
    with Boson() as camera:
        pn = camera.get_part_number()
        assert pn != ""

def test_ffc_request():
    with Boson() as camera:

        n_frames = camera.get_frame_count()
        print(n_frames)
        camera.do_ffc()
        time.sleep(1)
        ffc_frame = camera.get_last_ffc_frame_count()
        assert (ffc_frame-n_frames) < 200

def test_ffc_temperature():
    with Boson() as camera:
        assert camera.get_last_ffc_temperature() > 0

def test_frame_count():
    with Boson() as camera:
        assert camera.get_frame_count() > 0

def test_ffc_frame_threshold():
    with Boson() as camera:
        thresh = 6000
        camera.set_ffc_frame_threshold(thresh)
        assert camera.get_ffc_frame_threshold() == thresh

def test_ffc_temperature_threshold():
    with Boson() as camera:
        temp_diff = 5.0
        camera.set_ffc_temperature_threshold(temp_diff)
        assert camera.get_ffc_temperature_threshold() == temp_diff

def test_ffc_mode_manual():
    with Boson() as camera:

        camera.set_ffc_manual()
        mode = camera.get_ffc_mode()
        assert mode == flirpy.camera.boson.FLR_BOSON_MANUAL_FFC

def test_ffc_mode_auto():
    with Boson() as camera:

        camera.set_ffc_auto()
        mode = camera.get_ffc_mode()
        assert mode == flirpy.camera.boson.FLR_BOSON_AUTO_FFC


@pytest.mark.skipif(os.name != "nt", reason="Skipping Windows-only test")
def test_capture_windows():
    with Boson() as camera:
        # Currently have no way of figuring this out
        res = camera.grab(1)

        assert res is not None
        assert len(res.shape) == 2
        assert res.dtype == "uint16"

@pytest.mark.skipif(os.name == "nt", reason="Skipping on Windows")
def test_capture_unix():
    with Boson() as camera:
        res = camera.grab()

        assert res is not None
        assert len(res.shape) == 2
        assert res.dtype == "uint16"




import tau_config as ptc

import serial
import struct
import binascii
import time
import logging

import numpy as np
import tqdm
import math

# Tau Status codes

CAM_OK = 0x00
CAM_NOT_READY = 0x02
CAM_RANGE_ERROR = 0x03
CAM_UNDEFINED_ERROR = 0x04
CAM_UNDEFINED_PROCESS_ERROR = 0x05
CAM_UNDEFINED_FUNCTION_ERROR = 0x06
CAM_TIMEOUT_ERROR = 0x07
CAM_BYTE_COUNT_ERROR = 0x09
CAM_FEATURE_NOT_ENABLED= 0x0A

log = logging.getLogger()

class tau:

    def __enter__(self):
        return self
    
    def __exit__(self, type, value, traceback):
        self.conn.close()
        log.info("Disconnecting from camera.")

    def __init__(self, port, baud=921600):
        log.info("Connecting to camera.")
        self.conn = serial.Serial(port=port, baudrate=baud)

        if self.conn.is_open:
            log.info("Connected to camera at {}.".format(port))

            self.conn.flushInput()
            self.conn.flushOutput()
            self.conn.timeout = 1

            self.conn.read(self.conn.in_waiting)
        else:
            log.critical("Couldn't connect to camera!")
            raise IOError
            
    
    def ping(self):
        function = ptc.NO_OP

        self._send_packet(function)
        res = self._read_packet(function)

    def get_serial(self):
        function = ptc.SERIAL_NUMBER

        self._send_packet(function)
        res = self._read_packet(function)

        log.info("Camera serial: {}".format(int.from_bytes(res[7][:4], byteorder='big', signed=False)))
        log.info("Sensor serial: {}".format(int.from_bytes(res[7][4:], byteorder='big', signed=False)))

    def shutter_open(self):
        function = ptc.GET_SHUTTER_POSITION
        self._send_packet(function, "")
        res = self._read_packet(function)

        if int.from_bytes(res[7], byteorder='big', signed=False) == 0:
            return True
        else:
            return False
    
    def shutter_closed(self):
        return not self.shutter_open()

    def enable_test_pattern(self, mode=1):
        function = ptc.SET_TEST_PATTERN
        argument = struct.pack(">h", mode)
        self._send_packet(function, argument)
        time.sleep(0.2)
        res = self._read_packet(function)
    
    def disable_test_pattern(self):
        function = ptc.SET_TEST_PATTERN
        argument = struct.pack(">H", 0x00)
        self._send_packet(function, argument)
        time.sleep(0.2)
        res = self._read_packet(function)

    def get_core_status(self):
        function = ptc.READ_SENSOR_STATUS
        argument = struct.pack(">H", 0x0011)

        self._send_packet(function, argument)
        res = self._read_packet(function)

        status = struct.unpack(">H", res[7])[0]

        overtemp = status & (1 << 0)
        need_ffc = status & (1 << 2)
        gain_switch = status & (1 << 3)
        nuc_switch = status & (1 << 5)
        ffc = status & (1 << 6)

        if overtemp is not 0:
            log.critical("Core overtemperature warning! Remove power immediately!")
        
        if need_ffc is not 0:
            log.warning("Core desires a new flat field correction (FFC).")
        
        if gain_switch is not 0:
            log.warning("Core suggests that the gain be switched (check for over/underexposure).")
        
        if nuc_switch is not 0:
            log.warning("Core suggests that the NUC be switched.")

        if ffc is not 0:
            log.info("FFC is in progress.")

    def get_acceleration(self):
        function = ptc.READ_SENSOR_ACCELEROMETER
        argument = struct.pack(">H", 0x000B)

        self._send_packet(function, argument)
        res = self._read_packet(function)

        x, y, z  = struct.unpack(">HHHxx", res[7])

        x *= 0.1
        y *= 0.1
        z *= 0.1

        log.info("Acceleration: ({}, {}, {}) g".format(x, y, z))

        return (x, y, z)

    def get_fpa_temperature(self):
        function = ptc.READ_SENSOR_TEMPERATURE
        argument = struct.pack(">h", 0x00)
        self._send_packet(function, argument)
        res = self._read_packet(function)

        temperature = struct.unpack(">h", res[7])[0]
        temperature /= 10.0

        log.info("FPA temp: {}C".format(temperature))
        return temperature

    def get_housing_temperature(self):
        function = ptc.READ_SENSOR_TEMPERATURE
        argument = struct.pack(">h", 0x0A)
        self._send_packet(function, argument)
        time.sleep(1)
        res = self._read_packet(function)

        temperature = struct.unpack(">h", res[7])[0]
        temperature /= 100.0

        log.info("Housing temp: {}C".format(temperature))
        return temperature

    def close_shutter(self):
        function = ptc.SET_SHUTTER_POSITION
        argument = struct.pack(">h", 1)
        self._send_packet(function, argument)
        res = self._read_packet(function)

        return
    
    def open_shutter(self):
        function = ptc.SET_SHUTTER_POSITION
        argument = struct.pack(">h", 0)
        self._send_packet(function, argument)
        res = self._read_packet(function)

        return

    def _send_packet(self, command, argument=[]):

        packet_size = len(argument)
        assert(packet_size == command.cmd_bytes)  

        process_code = int(0x6E).to_bytes(1, 'big')
        status = int(0x00).to_bytes(1, 'big')
        function = command.code.to_bytes(1, 'big')

        # First CRC is the first 6 bytes of the packet
        # 1 - Process code
        # 2 - Status code
        # 3 - Reserved
        # 4 - Function
        # 5 - N Bytes MSB
        # 6 - N Bytes LSB

        packet = [process_code, status, function]
        packet.append( ((packet_size & 0xFF00) >> 8).to_bytes(1, 'big') )
        packet.append( (packet_size & 0x00FF).to_bytes(1, 'big') )
        crc_1 = binascii.crc_hqx(struct.pack("ccxccc", *packet), 0)

        packet.append( ((crc_1 & 0xFF00) >> 8).to_bytes(1, 'big') )
        packet.append( (crc_1 & 0x00FF).to_bytes(1, 'big') )

        if packet_size > 0:

            # Second CRC is the CRC of the data (if any)
            crc_2 = binascii.crc_hqx(argument, 0)
            packet.append(argument)
            packet.append( ((crc_2 & 0xFF00) >> 8).to_bytes(1, 'big') )
            packet.append( (crc_2 & 0x00FF).to_bytes(1, 'big') )

            fmt = ">cxcccccc{}scc".format(packet_size)
            
        else:
            fmt = ">cxccccccxxx"

        data = struct.pack(fmt, *packet)
        log.debug("Sending {}".format(data))

        self._send_data(data)
    
    def _check_header(self, data):

        res = struct.unpack(">BBxBBB", data)

        if res[0] != 0x6E:
            log.warning("Initial packet byte incorrect. Byte was: {}".format(res[0]))
            return False
        
        if not self.check_status(res[1]):
            return False
        
        return True

    def _read_packet(self, function, post_delay=0.1):
        argument_length = function.reply_bytes
        data = self.conn.read(10+argument_length)

        log.debug("Received: {}".format(data))

        if self._check_header(data[:6]) and len(data) > 0:
            if argument_length == 0:
                res = struct.unpack(">ccxcccccxx", data)
            else:
                res = struct.unpack(">ccxccccc{}scc".format(argument_length), data)
                #check_data_crc(res[7])
        else:
            res = None
            log.warning("Error reply from camera. Try re-sending command, or check parameters.")

        if post_delay > 0:
            time.sleep(post_delay)

        return res

    def check_status(self, code):

        if code == CAM_OK:
            log.debug("Response OK")
            return True
        elif code == CAM_BYTE_COUNT_ERROR:
            log.warning("Byte count error.")
        elif code == CAM_FEATURE_NOT_ENABLED:
            log.warning("Feature not enabled.")
        elif code == CAM_NOT_READY:
            log.warning("Camera not ready.")
        elif code == CAM_RANGE_ERROR:
            log.warning("Camera range error.")
        elif code == CAM_TIMEOUT_ERROR:
            log.warning("Camera timeout error.")
        elif code == CAM_UNDEFINED_ERROR:
            log.warning("Camera returned an undefined error.")
        elif code == CAM_UNDEFINED_FUNCTION_ERROR:
            log.warning("Camera function undefined. Check the function code.")
        elif code == CAM_UNDEFINED_PROCESS_ERROR:
            log.warning("Camera process undefined.")
        
        return False

    def get_num_snapshots(self):
        log.debug("Query snapshot status")
        function = ptc.GET_MEMORY_ADDRESS
        argument = struct.pack('>HH', 0xFFFE, 0x13)

        self._send_packet(function, argument)
        res = self._read_packet(function)
        snapshot_size, num_snapshots = struct.unpack(">ii", res[7])

        log.info("Used snapshot memory: {} Bytes".format(snapshot_size))
        log.info("Num snapshots: {}".format(num_snapshots))

        return num_snapshots, snapshot_size

    def erase_snapshots(self, frame_id=1):
        log.info("Erasing snapshots")

        num_snapshots, snapshot_used_memory = self.get_num_snapshots()

        if num_snapshots == 0:
            return

        # Get snapshot base address
        log.debug("Get capture address")
        function = ptc.GET_MEMORY_ADDRESS
        argument = struct.pack('>HH', 0xFFFF, 0x13)

        self._send_packet(function, argument)
        res = self._read_packet(function)
        snapshot_address, snapshot_area_size = struct.unpack(">ii", res[7])

        log.debug("Snapshot area size: {} Bytes".format(snapshot_area_size))

        # Get non-volatile memory base address
        function = ptc.GET_NV_MEMORY_SIZE
        argument = struct.pack('>H', 0xFFFF)

        self._send_packet(function, argument)
        res = self._read_packet(function)
        base_address, block_size = struct.unpack(">ii", res[7])

        # Compute the starting block
        starting_block = int((snapshot_address-base_address)/block_size)

        log.debug("Base address: {}".format(base_address))
        log.debug("Snapshot address: {}".format(snapshot_address))
        log.debug("Block size: {}".format(block_size))
        log.debug("Starting block: {}".format(starting_block))

        blocks_to_erase = math.ceil((snapshot_used_memory/block_size))

        log.debug("Number of blocks to erase: {}".format(blocks_to_erase))

        for i in range(blocks_to_erase):
            
            function = ptc.ERASE_BLOCK
            block_id = starting_block + i

            log.debug("Erasing block: {}".format(block_id))

            argument = struct.pack('>H', block_id)
            self._send_packet(function, argument)
            res = self._read_packet(function, post_delay=0.2)


    def snapshot(self, frame_id = 0):
        log.info("Capturing frame")

        self.get_core_status()

        if self.shutter_closed():
            log.warn("Shutter reports that it's closed. This frame may be corrupt!")

        function = ptc.TRANSFER_FRAME
        frame_code = 0x16
        argument = struct.pack('>BBH', frame_code, frame_id, 1)
        
        self._send_packet(function, argument)
        self._read_packet(function, post_delay=1)

        bytes_remaining = self.get_memory_status()
        log.info("{} bytes remaining to write.".format(bytes_remaining))

    def retrieve_snapshot(self, frame_id):
        # Get snapshot address
        log.info("Get capture address")
        function = ptc.GET_MEMORY_ADDRESS
        snapshot_memory = 0x13
        argument = struct.pack('>HH', frame_id, snapshot_memory)

        self._send_packet(function, argument)
        res = self._read_packet(function)
        snapshot_address, snapshot_size = struct.unpack(">ii", res[7])

        log.info("Snapshot size: {}".format(snapshot_size))

        n_transfers = math.ceil(snapshot_size/256)
        function = ptc.READ_MEMORY_256

        log.info("Reading frame {} ({} bytes)".format(frame_id, snapshot_size))
        # For N reads, read data
        data = []
        remaining = snapshot_size
        for i in tqdm.tqdm(range(n_transfers)):

            n_bytes = min(remaining, 256)
            function.reply_bytes = n_bytes

            argument = struct.pack('>iH', snapshot_address+i*256, n_bytes)
            self._send_packet(function, argument)
            res = self._read_packet(function, post_delay=0)

            data += struct.unpack(">{}B".format(int(n_bytes)), res[7])
            remaining -= n_bytes

        image = np.array(data, dtype='uint8')

        return image
    
    def get_memory_status(self):
        function = ptc.MEMORY_STATUS
    
        self._send_packet(function)
        res = self._read_packet(function)

        remaining_bytes = struct.unpack(">H", res[7])[0]

        if remaining_bytes == 0xFFFF:
            log.warn("Erase error")
        elif remaining_bytes == 0xFFFE:
            log.warn("Write error")
        else:
            return remaining_bytes
    
    def ffc(self):
        function = ptc.DO_FFC_SHORT

        self._send_packet(function)
        res = self._read_packet(function)

    def get_last_image(self):
        num_snapshots, _ = self.get_num_snapshots()
        
        if num_snapshots > 0:
            return self.retrieve_snapshot(num_snapshots - 1)
        else:
            return None
    
    def _send_data(self, data):
        nbytes = self.conn.write(data)
        self.conn.flush()
        return
    
    def _recieve_data(self, nbytes):
        return self.conn.read(nbytes)



import serial
import binascii
import time

class Core:

    def __init__(self):
        self.conn = None
    
    def connect(self, port, baudrate):
        self.conn = serial.Serial(port, baudrate, timeout=2)
        self.conn.read_all()
    
    def send(self, packet):
        return self.conn.write(packet)
    
    def receive(self, nbytes):
        frame = self.conn.read(nbytes)
        return frame
    
    def grab(self):
        pass
    
    def disconnect(self):
        if self.conn is not None:
            if self.conn.is_open:
                self.conn.close()
    
    def close(self):
        self.disconnect()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()


import re
import struct
import numpy as np
import sys
import six

from flirpy.util.raw import raw2temp
from flirpy.util.exiftool import Exiftool

class Fff:

    def __init__(self, data, height = 512, width = 640, exiftool_path=None):

        self.height = height
        self.width = width
        self.image = None
        self.filename = None

        self.exiftool = Exiftool(exiftool_path)

        # This is a dirty hack to force Python 2 to recognise whether
        # it's a filename or a file. An FFF file is almost always
        # larger than 2kB.
        if isinstance(data, str) and len(data) < 4096:
            with open(data, 'rb') as fff_file:
                self.filename = fff_file
                self.data = fff_file.read()
        elif isinstance(data, bytes):
            self.data = data
        else:
            raise TypeError("Data should be a bytes object or a string filename")

    def write(self, path):
        with open(path, 'wb') as fff_file:
            fff_file.write(self.data)

    def _find_data_offset(self, data):
    
        search =  struct.pack("<H", self.width-1)\
                    +b"\x00\x00"\
                    + struct.pack("<H", self.height-1)

        valid = re.compile(search)
        res = valid.search(data)

        return res.end()+14
    
    def get_radiometric_image(self, meta):
        image = raw2temp(self.get_image(), meta)

        return image

    def get_image(self):
        
        if self.image is None:
            offset = self._find_data_offset(self.data)
            count = self.height*self.width
            self.image = np.frombuffer(self.data, offset=offset, dtype='uint16', count=count).reshape((self.height, self.width))
        
        return self.image
    
    def get_gps(self):
        valid = re.compile("[0-9]{4}[NS]\x00[EW]\x00".encode())

        print(valid)
        print(self.data)

        res = valid.search(self.data)
        print(res)
        start_pos = res.start()

        s = struct.Struct("<4xcxcx4xddf32xcxcx4xff")

        return s.unpack_from(self.data, start_pos)


import os
import sys
import pkg_resources
import subprocess
import glob
import platform

import logging
logger = logging.getLogger(__name__)

class Exiftool:

    def __init__(self, path=None):
        
        if path is None:
            if sys.platform.startswith('win32'):
                self.path = pkg_resources.resource_filename('flirpy', 'bin/exiftool.exe')
            # Fix problems on ARM platforms
            elif platform.uname()[4].startswith("arm"):
                if os.path.isfile("/usr/bin/exiftool"):
                    self.path = "/usr/bin/exiftool"
                else:
                    logger.warning("Exiftool not installed, try: apt install exiftool")
            else:
                self.path = pkg_resources.resource_filename('flirpy', 'bin/exiftool')

        else:
            self.path = path
            self._check_path()
    
    def _check_path(self):
        try:
            subprocess.check_output([self.path])
            logger.info("Exiftool path verified at {}".format(self.path))
            return True
        except FileNotFoundError:
            logger.error("Couldn't find Exiftool at {}".format(self.path))
            return False

        return False

    def copy_meta(self, folder, filemask="%f.fff", output_folder="./", ext="tiff"):

        cwd = folder

        cmd = [self.path]
        cmd.append("-r")
        cmd.append("-overwrite_original")
        cmd.append("-tagsfromfile")
        cmd.append(filemask)
        cmd.append("-ext")
        cmd.append(ext)
        cmd.append(output_folder)

        logger.debug(" ".join(cmd))

        res = subprocess.call(cmd, cwd=cwd, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        return res
    
    def write_meta(self, filemask):

        # Do some mangling here to avoid busting the command line limit.
        # First, we run the command in the right working directory
        cwd, _ = os.path.split(filemask)

        # Then we expand the wildcard and pass to the shell 
        # (but don't need to pass the full path since we use cwd)
        files = glob.glob(filemask)
        files = [os.path.split(f)[1] for f in files]

        cmd = [self.path]
        cmd += files
        cmd.append("-w")
        cmd.append(".txt")

        logger.debug(" ".join(cmd))

        res = subprocess.call(cmd, cwd=cwd, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        
        return res
    
    def meta_from_file(self, filename):
        meta = {}

        with open(filename, 'r') as f:
            for line in f:
                res = line.split(":")

                key = res[0].strip()
                value = "".join(res[1:])

                meta[key] = value

        return meta


import pkg_resources
import os
import subprocess

import sys
if sys.version.startswith('2'):
    from backports import tempfile
else:
    import tempfile
import glob
import cv2

from flirpy.io.seq import splitter

class TestSeqSplit:
    def setup_method(self):
        self.tempdir = tempfile.TemporaryDirectory()
        self.output_folder = self.tempdir.name

        folder = os.path.dirname(os.path.abspath(__file__))
        self.test_data_path = os.path.join(folder, "test_data", "test.seq")

    def teardown_method(self):
        self.tempdir.cleanup()

    def test_output_folder(self):
        self.sp = splitter(self.output_folder)
        assert os.path.exists(self.output_folder)

    def test_process_seq_split(self):
        self.sp = splitter(self.output_folder)

        self.sp.split_folders = True
        self.sp.process(self.test_data_path)

        assert os.path.exists(os.path.join(self.output_folder, "test", "raw"))
        assert os.path.exists(os.path.join(self.output_folder, "test", "preview"))
        assert os.path.exists(os.path.join(self.output_folder, "test", "radiometric"))

        assert len(glob.glob(os.path.join(self.output_folder, "test", "raw", "*.fff"))) > 0
        assert len(glob.glob(os.path.join(self.output_folder, "test", "raw", "*.txt"))) > 0
        assert len(glob.glob(os.path.join(self.output_folder, "test", "preview", "*.png"))) > 0
        assert len(glob.glob(os.path.join(self.output_folder, "test", "radiometric", "*.tiff"))) > 0

    def test_raw_is_16_bit(self):
        self.sp = splitter(self.output_folder)
        self.sp.process(self.test_data_path)

        raw_files = glob.glob(os.path.join(self.output_folder, "test", "radiometric", "*.tiff"))

        for raw_file in raw_files:
            assert(cv2.imread(raw_file, cv2.IMREAD_UNCHANGED).dtype == "uint16")


    def test_process_no_mmap(self):
        self.sp = splitter(self.output_folder)
        self.sp.use_mmap = False
        self.sp.process(self.test_data_path)

    def test_process_seq_no_split(self):
        self.sp = splitter(self.output_folder)

        self.sp.split_folders = False
        self.sp.process(self.test_data_path)

        assert os.path.exists(os.path.join(self.output_folder, "test", "raw")) is False
        assert os.path.exists(os.path.join(self.output_folder, "test", "preview")) is False
        assert os.path.exists(os.path.join(self.output_folder, "test", "radiometric")) is False
        
        assert len(glob.glob(os.path.join(self.output_folder, "test", "*.fff"))) > 0
        assert len(glob.glob(os.path.join(self.output_folder, "test", "*.txt"))) > 0
        assert len(glob.glob(os.path.join(self.output_folder, "test", "*.png"))) > 0
        assert len(glob.glob(os.path.join(self.output_folder, "test", "*.tiff"))) > 0



from setuptools import setup, find_packages

import os
import sys
import platform

if platform.uname()[4].startswith("arm") and sys.version.startswith("2"):
    install_requires=['pyserial', 'tqdm', 'numpy']
else:
    install_requires=['pyserial', 'opencv-python-headless', 'tqdm', 'numpy']

__packagename__ = "flirpy"

setup(
    name=__packagename__,
    version='0.0.4',
    packages=find_packages(),
    author='Josh Veitch-Michaelis',
    author_email='j.veitchmichaelis@gmail.com',
    license='MIT',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    zip_safe=False,
    include_package_data=True,
    scripts=['scripts/split_seqs'],
    install_requires = install_requires
)


import numpy as np
import math

def raw2temp(raw, meta):
        ATA1 = float(meta["Atmospheric Trans Alpha 1"])
        ATA2 = float(meta["Atmospheric Trans Alpha 2"])
        ATB1 = float(meta["Atmospheric Trans Beta 1"])
        ATB2 = float(meta["Atmospheric Trans Beta 2"])
        ATX = float(meta["Atmospheric Trans X"])
        PR1 = float(meta["Planck R1"])
        PR2 = float(meta["Planck R2"])
        PO = float(meta["Planck O"])
        PB = float(meta["Planck B"])
        PF = float(meta["Planck F"])
        E = float(meta["Emissivity"])
        IRT = float(meta["IR Window Transmission"])
        IRWTemp = float(meta["IR Window Temperature"].split('C')[0])
        OD = float(meta["Object Distance"].split('m')[0])
        ATemp = float(meta["Atmospheric Temperature"].split('C')[0])
        RTemp = float(meta["Reflected Apparent Temperature"].split('C')[0])
        humidity = float(meta["Relative Humidity"].split('%')[0])

      # Equations to conert to temperature
      # See http://130.15.24.88/exiftool/forum/index.php/topic,4898.60.html
      # Standard equation: temperature<-PB/log(PR1/(PR2*(raw+PO))+PF)-273.15
      # Other source of information: Minkina and Dudzik's Infrared Thermography: Errors and Uncertainties

        window_emissivity = 1 - IRT
        window_reflectivity = 0

        # Converts relative humidity into water vapour pressure (mmHg)
        water = (humidity/100.0)*math.exp(1.5587+0.06939*(ATemp)-0.00027816*(ATemp)**2+0.00000068455*(ATemp)**3)

        #tau1 = ATX*np.exp(-np.sqrt(OD/2))
        tau1 = ATX*np.exp(-np.sqrt(OD/2)*(ATA1+ATB1*np.sqrt(water)))+(1-ATX)*np.exp(-np.sqrt(OD/2)*(ATA2+ATB2*np.sqrt(water)))
        tau2 = ATX*np.exp(-np.sqrt(OD/2)*(ATA1+ATB1*np.sqrt(water)))+(1-ATX)*np.exp(-np.sqrt(OD/2)*(ATA2+ATB2*np.sqrt(water)))

        # transmission through atmosphere - equations from Minkina and Dudzik's Infrared Thermography Book
        # Note: for this script, we assume the thermal window is at the mid-point (OD/2) between the source
        # and the camera sensor

        raw_refl = PR1/(PR2*(np.exp(PB/(RTemp+273.15))-PF))-PO   # radiance reflecting off the object before the window
        raw_refl_attn = (1-E)/E*raw_refl   # attn = the attenuated radiance (in raw units) 

        raw_atm1 = PR1/(PR2*(np.exp(PB/(ATemp+273.15))-PF))-PO # radiance from the atmosphere (before the window)
        raw_atm1_attn = (1-tau1)/E/tau1*raw_atm1 # attn = the attenuated radiance (in raw units) 

        raw_window = PR1/(PR2*(math.exp(PB/(IRWTemp+273.15))-PF))-PO
        raw_window_attn = window_emissivity/E/tau1/IRT*raw_window

        raw_refl2 = PR1/(PR2*(np.exp(PB/(RTemp+273.15))-PF))-PO   
        raw_refl2_attn = window_reflectivity/E/tau1/IRT*raw_refl2

        raw_atm2 = PR1/(PR2*(np.exp(PB/(ATemp+273.15))-PF))-PO
        raw_atm2_attn = (1-tau2)/E/tau1/IRT/tau2*raw_atm2

        raw_object = raw/E/tau1/IRT/tau2-raw_atm1_attn-raw_atm2_attn-raw_window_attn-raw_refl_attn-raw_refl2_attn

        temp = PB/np.log(PR1/(PR2*(raw+PO))+PF)-273.15

        return temp